@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/domain.c
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.7.2   2010/04/01
+ * Version: 1.7.3   2011/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -34,38 +34,30 @@ LIST_HEAD(ccs_domain_list);
  * ccs_audit_execute_handler_log - Audit execute_handler log.
  *
  * @ee:         Pointer to "struct ccs_execve_entry".
- * @is_default: True if it is "execute_handler" log.
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int ccs_audit_execute_handler_log(struct ccs_execve_entry *ee,
-					 const bool is_default)
+static int ccs_audit_execute_handler_log(struct ccs_execve_entry *ee)
 {
-	int error;
 	struct ccs_request_info *r = &ee->r;
-	struct ccs_domain_info *domain = r->domain;
 	const char *handler = ee->handler->name;
+	r->type = CCS_MAC_FILE_EXECUTE;
 	r->mode = ccs_get_mode(r->profile, CCS_MAC_FILE_EXECUTE);
-	r->domain = ee->previous_domain;
-	error = ccs_write_audit_log(true, r, "%s %s\n",
-				    is_default ? CCS_KEYWORD_EXECUTE_HANDLER :
-				    CCS_KEYWORD_DENIED_EXECUTE_HANDLER,
-				    handler);
-	r->domain = domain;
-	return error;
+	return ccs_write_audit_log(true, r, "%s" CCS_KEYWORD_EXECUTE_HANDLER
+				   " %s\n", ee->handler_type ==
+				   CCS_TYPE_DENIED_EXECUTE_HANDLER ?
+				   "denied_" : "", handler);
 }
 
 /**
  * ccs_audit_domain_creation_log - Audit domain creation log.
  *
- * @domain:  Pointer to "struct ccs_domain_info".
- *
  * Returns 0 on success, negative value otherwise.
  */
-static int ccs_audit_domain_creation_log(struct ccs_domain_info *domain)
+static int ccs_audit_domain_creation_log(void)
 {
 	struct ccs_request_info r;
-	ccs_init_request_info(&r, domain, CCS_MAC_FILE_EXECUTE);
+	ccs_init_request_info(&r, CCS_MAC_FILE_EXECUTE);
 	return ccs_write_audit_log(false, &r, "use_profile %u\n", r.profile);
 }
 
@@ -573,13 +565,16 @@ static int ccs_find_next_domain(struct ccs_execve_entry *ee)
 	struct ccs_request_info *r = &ee->r;
 	const struct ccs_path_info *handler = ee->handler;
 	struct ccs_domain_info *domain = NULL;
-	const char *old_domain_name = r->domain->domainname->name;
+	struct ccs_domain_info * const old_domain = ccs_current_domain();
+	const char *old_domain_name = old_domain->domainname->name;
 	struct linux_binprm *bprm = ee->bprm;
-	const u32 ccs_flags = current->ccs_flags;
+	struct task_struct *task = current;
+	const u32 ccs_flags = task->ccs_flags;
 	struct ccs_path_info rn = { }; /* real name */
 	struct ccs_path_info ln; /* last name */
 	int retval;
 	bool need_kfree = false;
+	bool domain_created = false;
 	ln.name = ccs_last_word(old_domain_name);
 	ccs_fill_path_info(&ln);
  retry:
@@ -630,20 +625,20 @@ static int ccs_find_next_domain(struct ccs_execve_entry *ee)
 	}
 
 	/* Calculate domain to transit to. */
-	if (ccs_is_domain_initializer(r->domain->domainname, &rn, &ln)) {
+	if (ccs_is_domain_initializer(old_domain->domainname, &rn, &ln)) {
 		/* Transit to the child of ccs_kernel_domain domain. */
 		snprintf(ee->tmp, CCS_EXEC_TMPSIZE - 1, ROOT_NAME " " "%s",
 			 rn.name);
-	} else if (r->domain == &ccs_kernel_domain && !ccs_policy_loaded) {
+	} else if (old_domain == &ccs_kernel_domain && !ccs_policy_loaded) {
 		/*
 		 * Needn't to transit from kernel domain before starting
 		 * /sbin/init. But transit from kernel domain if executing
 		 * initializers because they might start before /sbin/init.
 		 */
-		domain = r->domain;
-	} else if (ccs_is_domain_keeper(r->domain->domainname, &rn, &ln)) {
+		domain = old_domain;
+	} else if (ccs_is_domain_keeper(old_domain->domainname, &rn, &ln)) {
 		/* Keep current domain. */
-		domain = r->domain;
+		domain = old_domain;
 	} else {
 		/* Normal domain transition. */
 		snprintf(ee->tmp, CCS_EXEC_TMPSIZE - 1, "%s %s",
@@ -664,12 +659,12 @@ static int ccs_find_next_domain(struct ccs_execve_entry *ee)
 	}
 	domain = ccs_find_or_assign_new_domain(ee->tmp, r->profile);
 	if (domain)
-		ccs_audit_domain_creation_log(domain);
+		domain_created = true;
  done:
 	if (!domain) {
 		retval = (r->mode == CCS_CONFIG_ENFORCING) ? -EPERM : 0;
-		if (!r->domain->domain_transition_failed) {
-			r->domain->domain_transition_failed = true;
+		if (!old_domain->domain_transition_failed) {
+			old_domain->domain_transition_failed = true;
 			ccs_write_audit_log(false, r,
 					    CCS_KEYWORD_TRANSITION_FAILED
 					    "\n");
@@ -679,9 +674,29 @@ static int ccs_find_next_domain(struct ccs_execve_entry *ee)
 	} else {
 		retval = 0;
 	}
- out:
+	if (!retval && handler)
+		ccs_audit_execute_handler_log(ee);
+	/*
+	 * Tell GC that I started execve().
+	 * Also, tell open_exec() to check read permission.
+	 */
+	task->ccs_flags |= CCS_TASK_IS_IN_EXECVE;
+	/*
+	 * Make task->ccs_flags visible to GC before changing
+	 * task->ccs_domain_info .
+	 */
+	smp_mb();
+	/*
+	 * Proceed to the next domain in order to allow reaching via PID.
+	 * It will be reverted if execve() failed. Reverting is not good.
+	 * But it is better than being unable to reach via PID in interactive
+	 * enforcing mode.
+	 */
 	if (domain)
-		r->domain = domain;
+		task->ccs_domain_info = domain;
+	if (domain_created)
+		ccs_audit_domain_creation_log();
+ out:
 	if (need_kfree)
 		kfree(rn.name);
 	return retval;
@@ -708,6 +723,9 @@ static int ccs_environ(struct ccs_execve_entry *ee)
 	int envp_count = bprm->envc;
 	/* printk(KERN_DEBUG "start %d %d\n", argv_count, envp_count); */
 	int error = -ENOMEM;
+	ee->r.type = CCS_MAC_ENVIRON;
+	ee->r.mode = ccs_get_mode(ccs_current_domain()->profile,
+				  CCS_MAC_ENVIRON);
 	if (!r->mode || !envp_count)
 		return 0;
 	arg_ptr = kzalloc(CCS_EXEC_TMPSIZE, CCS_GFP_FLAGS);
@@ -807,67 +825,23 @@ static void ccs_unescape(unsigned char *dest)
 	*dest = '\0';
 }
 
-/**
- * ccs_root_depth - Get number of directories to strip.
- *
- * @dentry: Pointer to "struct dentry".
- * @vfsmnt: Pointer to "struct vfsmount".
- *
- * Returns number of directories to strip.
- */
-static inline int ccs_root_depth(struct dentry *dentry, struct vfsmount *vfsmnt)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
+static int ccs_copy_argv(const char *arg, struct linux_binprm *bprm)
 {
-	int depth = 0;
-	ccs_realpath_lock();
-	for (;;) {
-		if (dentry == vfsmnt->mnt_root || IS_ROOT(dentry)) {
-			/* Global root? */
-			if (vfsmnt->mnt_parent == vfsmnt)
-				break;
-			dentry = vfsmnt->mnt_mountpoint;
-			vfsmnt = vfsmnt->mnt_parent;
-			continue;
-		}
-		dentry = dentry->d_parent;
-		depth++;
-	}
-	ccs_realpath_unlock();
-	return depth;
+	const int ret = copy_strings_kernel(1, &arg, bprm);
+	if (ret >= 0)
+		bprm->argc++;
+	return ret;
 }
-
-/**
- * ccs_get_root_depth - return the depth of root directory.
- *
- * Returns number of directories to strip.
- */
-static int ccs_get_root_depth(void)
+#else
+static int ccs_copy_argv(char *arg, struct linux_binprm *bprm)
 {
-	int depth;
-	struct dentry *dentry;
-	struct vfsmount *vfsmnt;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-	struct path root;
-#endif
-	read_lock(&current->fs->lock);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-	root = current->fs->root;
-	path_get(&current->fs->root);
-	dentry = root.dentry;
-	vfsmnt = root.mnt;
-#else
-	dentry = dget(current->fs->root);
-	vfsmnt = mntget(current->fs->rootmnt);
-#endif
-	read_unlock(&current->fs->lock);
-	depth = ccs_root_depth(dentry, vfsmnt);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-	path_put(&root);
-#else
-	dput(dentry);
-	mntput(vfsmnt);
-#endif
-	return depth;
+	const int ret = copy_strings_kernel(1, &arg, bprm);
+	if (ret >= 0)
+		bprm->argc++;
+	return ret;
 }
+#endif
 
 /**
  * ccs_try_alt_exec - Try to start execute handler.
@@ -938,35 +912,29 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 
 	/* Set argv[6] */
 	{
-		char *cp = ee->tmp;
 		snprintf(ee->tmp, CCS_EXEC_TMPSIZE - 1, "%d", original_envc);
-		retval = copy_strings_kernel(1, &cp, bprm);
+		retval = ccs_copy_argv(ee->tmp, bprm);
 		if (retval < 0)
 			goto out;
-		bprm->argc++;
 	}
 
 	/* Set argv[5] */
 	{
-		char *cp = ee->tmp;
 		snprintf(ee->tmp, CCS_EXEC_TMPSIZE - 1, "%d", original_argc);
-		retval = copy_strings_kernel(1, &cp, bprm);
+		retval = ccs_copy_argv(ee->tmp, bprm);
 		if (retval < 0)
 			goto out;
-		bprm->argc++;
 	}
 
 	/* Set argv[4] */
 	{
-		retval = copy_strings_kernel(1, &bprm->filename, bprm);
+		retval = ccs_copy_argv(bprm->filename, bprm);
 		if (retval < 0)
 			goto out;
-		bprm->argc++;
 	}
 
 	/* Set argv[3] */
 	{
-		char *cp = ee->tmp;
 		const u32 ccs_flags = task->ccs_flags;
 		snprintf(ee->tmp, CCS_EXEC_TMPSIZE - 1,
 			 "pid=%d uid=%d gid=%d euid=%d egid=%d suid=%d "
@@ -978,67 +946,82 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 			 current_fsuid(), current_fsgid(),
 			 (u8) (ccs_flags >> 24), (u8) (ccs_flags >> 16),
 			 (u8) (ccs_flags >> 8));
-		retval = copy_strings_kernel(1, &cp, bprm);
+		retval = ccs_copy_argv(ee->tmp, bprm);
 		if (retval < 0)
 			goto out;
-		bprm->argc++;
 	}
 
 	/* Set argv[2] */
 	{
 		char *exe = (char *) ccs_get_exe();
 		if (exe) {
-			retval = copy_strings_kernel(1, &exe, bprm);
+			retval = ccs_copy_argv(exe, bprm);
 			kfree(exe);
 		} else {
-			exe = ee->tmp;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
+			retval = ccs_copy_argv("<unknown>", bprm);
+#else
 			snprintf(ee->tmp, CCS_EXEC_TMPSIZE - 1, "<unknown>");
-			retval = copy_strings_kernel(1, &exe, bprm);
+			retval = ccs_copy_argv(ee->tmp, bprm);
+#endif
 		}
 		if (retval < 0)
 			goto out;
-		bprm->argc++;
 	}
 
 	/* Set argv[1] */
 	{
-		char *cp = ee->tmp;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
+		retval = ccs_copy_argv(ccs_current_domain()->domainname->name,
+				       bprm);
+#else
 		snprintf(ee->tmp, CCS_EXEC_TMPSIZE - 1, "%s",
 			 ccs_current_domain()->domainname->name);
-		retval = copy_strings_kernel(1, &cp, bprm);
+		retval = ccs_copy_argv(ee->tmp, bprm);
+#endif
 		if (retval < 0)
 			goto out;
-		bprm->argc++;
 	}
 
 	/* Set argv[0] */
 	{
-		int depth = ccs_get_root_depth();
-		int len = ee->handler->total_len + 1;
-		char *cp = kmalloc(len, CCS_GFP_FLAGS);
+		struct path root;
+		char *cp;
+		int root_len;
+		int handler_len;
+		get_fs_root(current->fs, &root);
+		cp = ccs_realpath_from_path(&root);
+		path_put(&root);
 		if (!cp) {
 			retval = -ENOMEM;
 			goto out;
 		}
+		root_len = strlen(cp);
+		retval = strncmp(ee->handler->name, cp, root_len);
+		root_len--;
+		kfree(cp);
+		if (retval) {
+			retval = -ENOENT;
+			goto out;
+		}
+		handler_len = ee->handler->total_len + 1;
+		cp = kmalloc(handler_len, CCS_GFP_FLAGS);
+		if (!cp) {
+			retval = -ENOMEM;
+			goto out;
+		}
+		/* ee->handler_path is released by ccs_finish_execve(). */
 		ee->handler_path = cp;
-		memmove(cp, ee->handler->name, len);
+		/* Adjust root directory for open_exec(). */
+		memmove(cp, ee->handler->name + root_len,
+			handler_len - root_len);
 		ccs_unescape(cp);
 		retval = -ENOENT;
 		if (!*cp || *cp != '/')
 			goto out;
-		/* Adjust root directory for open_exec(). */
-		while (depth) {
-			cp = strchr(cp + 1, '/');
-			if (!cp)
-				goto out;
-			depth--;
-		}
-		memmove(ee->handler_path, cp, strlen(cp) + 1);
-		cp = ee->handler_path;
-		retval = copy_strings_kernel(1, &cp, bprm);
+		retval = ccs_copy_argv(cp, bprm);
 		if (retval < 0)
 			goto out;
-		bprm->argc++;
 	}
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23)
 #if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 24)
@@ -1046,7 +1029,9 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 #endif
 #endif
 
-	/* OK, now restart the process with execute handler program's dentry. */
+	/*
+	 * OK, now restart the process with execute handler program's dentry.
+	 */
 	filp = open_exec(ee->handler_path);
 	if (IS_ERR(filp)) {
 		retval = PTR_ERR(filp);
@@ -1083,7 +1068,7 @@ static bool ccs_find_execute_handler(struct ccs_execve_entry *ee,
 				     const u8 type)
 {
 	struct task_struct *task = current;
-	const struct ccs_domain_info *domain = ccs_current_domain();
+	const struct ccs_domain_info * const domain = ccs_current_domain();
 	struct ccs_acl_info *ptr;
 	bool found = false;
 	/*
@@ -1092,13 +1077,14 @@ static bool ccs_find_execute_handler(struct ccs_execve_entry *ee,
 	 */
 	if (task->ccs_flags & CCS_TASK_IS_EXECUTE_HANDLER)
 		return false;
-	list_for_each_entry(ptr, &domain->acl_info_list, list) {
+	list_for_each_entry_rcu(ptr, &domain->acl_info_list, list) {
 		struct ccs_execute_handler_record *acl;
 		if (ptr->type != type)
 			continue;
 		acl = container_of(ptr, struct ccs_execute_handler_record,
 				   head);
 		ee->handler = acl->handler;
+		ee->handler_type = type;
 		found = true;
 		break;
 	}
@@ -1146,7 +1132,8 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 		 */
 		char *kaddr = kmap_atomic(page, KM_USER0);
 		dump->page = page;
-		memcpy(dump->data + offset, kaddr + offset, PAGE_SIZE - offset);
+		memcpy(dump->data + offset, kaddr + offset,
+		       PAGE_SIZE - offset);
 		kunmap_atomic(kaddr, KM_USER0);
 	}
 	/* Same with put_arg_page(page) in fs/exec.c */
@@ -1189,52 +1176,26 @@ static int ccs_start_execve(struct linux_binprm *bprm,
 	/* Clear manager flag. */
 	task->ccs_flags &= ~CCS_TASK_IS_POLICY_MANAGER;
 	*eep = ee;
-	ccs_init_request_info(&ee->r, NULL, CCS_MAC_FILE_EXECUTE);
+	ccs_init_request_info(&ee->r, CCS_MAC_FILE_EXECUTE);
 	ee->r.ee = ee;
 	ee->bprm = bprm;
 	ee->r.obj = &ee->obj;
 	ee->obj.path1.dentry = bprm->file->f_dentry;
 	ee->obj.path1.mnt = bprm->file->f_vfsmnt;
-	if (ccs_find_execute_handler(ee, CCS_TYPE_EXECUTE_HANDLER)) {
-		retval = ccs_try_alt_exec(ee);
-		if (!retval)
-			ccs_audit_execute_handler_log(ee, true);
-		goto ok;
-	}
+	/*
+	 * No need to call ccs_environ() for execute handler because envp[] is
+	 * moved to argv[].
+	 */
+	if (ccs_find_execute_handler(ee, CCS_TYPE_EXECUTE_HANDLER))
+		return ccs_try_alt_exec(ee);
 	retval = ccs_find_next_domain(ee);
-	if (retval != -EPERM)
-		goto ok;
-	if (ccs_find_execute_handler(ee, CCS_TYPE_DENIED_EXECUTE_HANDLER)) {
-		retval = ccs_try_alt_exec(ee);
-		if (!retval)
-			ccs_audit_execute_handler_log(ee, false);
+	if (retval == -EPERM) {
+		if (ccs_find_execute_handler(ee,
+					     CCS_TYPE_DENIED_EXECUTE_HANDLER))
+			return ccs_try_alt_exec(ee);
 	}
- ok:
-	if (retval < 0)
-		goto out;
-	/*
-	 * Tell GC that I started execve().
-	 * Also, tell open_exec() to check read permission.
-	 */
-	task->ccs_flags |= CCS_TASK_IS_IN_EXECVE;
-	/*
-	 * Make task->ccs_flags visible to GC before changing
-	 * task->ccs_domain_info .
-	 */
-	smp_mb();
-	/*
-	 * Proceed to the next domain in order to allow reaching via PID.
-	 * It will be reverted if execve() failed. Reverting is not good.
-	 * But it is better than being unable to reach via PID in interactive
-	 * enforcing mode.
-	 */
-	task->ccs_domain_info = ee->r.domain;
-	ee->r.mode = ccs_get_mode(ee->r.domain->profile, CCS_MAC_ENVIRON);
-	retval = ccs_environ(ee);
-	if (retval < 0)
-		goto out;
-	retval = 0;
- out:
+ 	if (!retval)
+		retval = ccs_environ(ee);
 	return retval;
 }
 
@@ -1291,10 +1252,11 @@ int ccs_may_transit(const char *domainname, const char *pathname)
 	struct ccs_request_info r;
 	struct ccs_domain_info *domain;
 	int error;
+	bool domain_created = false;
 	name.name = pathname;
 	ccs_fill_path_info(&name);
 	/* Check allow_transit permission. */
-	ccs_init_request_info(&r, NULL, CCS_MAC_FILE_TRANSIT);
+	ccs_init_request_info(&r, CCS_MAC_FILE_TRANSIT);
 	error = ccs_path_permission(&r, CCS_TYPE_TRANSIT, &name);
 	if (error)
 		return error;
@@ -1304,11 +1266,13 @@ int ccs_may_transit(const char *domainname, const char *pathname)
 	    strlen(domainname) < CCS_EXEC_TMPSIZE - 10) {
 		domain = ccs_find_or_assign_new_domain(domainname, r.profile);
 		if (domain)
-			ccs_audit_domain_creation_log(domain);
+			domain_created = true;
 	}
 	if (domain) {
 		error = 0;
 		current->ccs_domain_info = domain;
+		if (domain_created)
+			ccs_audit_domain_creation_log();
 	} else {
 		error = -ENOENT;
 	}
